@@ -13,23 +13,27 @@ use super::{UnknownUnit, Angle};
 #[cfg(feature = "mint")]
 use mint;
 use num::{One, Zero};
-use point::TypedPoint2D;
-use vector::{TypedVector2D, vec2};
-use rect::TypedRect;
-use transform3d::TypedTransform3D;
+use point::{Point2D, point2};
+use vector::{Vector2D, vec2};
+use rect::Rect;
+use transform3d::Transform3D;
 use core::ops::{Add, Mul, Div, Sub, Neg};
 use core::marker::PhantomData;
+use core::cmp::{Eq, PartialEq};
+use core::hash::{Hash};
 use approxeq::ApproxEq;
 use trig::Trig;
 use core::fmt;
 use num_traits::NumCast;
+#[cfg(feature = "serde")]
+use serde;
 
 /// A 2d transform stored as a 3 by 2 matrix in row-major order in memory.
 ///
 /// Transforms can be parametrized over the source and destination units, to describe a
 /// transformation from a space to another.
-/// For example, `TypedTransform2D<f32, WorldSpace, ScreenSpace>::transform_point4d`
-/// takes a `TypedPoint2D<f32, WorldSpace>` and returns a `TypedPoint2D<f32, ScreenSpace>`.
+/// For example, `Transform2D<f32, WorldSpace, ScreenSpace>::transform_point4d`
+/// takes a `Point2D<f32, WorldSpace>` and returns a `Point2D<f32, ScreenSpace>`.
 ///
 /// Transforms expose a set of convenience methods for pre- and post-transformations.
 /// A pre-transformation corresponds to adding an operation that is applied before
@@ -40,8 +44,7 @@ use num_traits::NumCast;
 /// a vector is `v * T`. If your library is using column vectors, use `row_major` functions when you
 /// are asked for `column_major` representations and vice versa.
 #[repr(C)]
-#[derive(EuclidMatrix)]
-pub struct TypedTransform2D<T, Src, Dst> {
+pub struct Transform2D<T, Src, Dst> {
     pub m11: T, pub m12: T,
     pub m21: T, pub m22: T,
     pub m31: T, pub m32: T,
@@ -49,17 +52,94 @@ pub struct TypedTransform2D<T, Src, Dst> {
     pub _unit: PhantomData<(Src, Dst)>,
 }
 
-/// The default 2d transform type with no units.
-pub type Transform2D<T> = TypedTransform2D<T, UnknownUnit, UnknownUnit>;
+impl<T: Copy, Src, Dst> Copy for Transform2D<T, Src, Dst> {}
 
-impl<T: Copy, Src, Dst> TypedTransform2D<T, Src, Dst> {
+impl<T: Clone, Src, Dst> Clone for Transform2D<T, Src, Dst> {
+    fn clone(&self) -> Self {
+        Transform2D {
+            m11: self.m11.clone(),
+            m12: self.m12.clone(),
+            m21: self.m21.clone(),
+            m22: self.m22.clone(),
+            m31: self.m31.clone(),
+            m32: self.m32.clone(),
+            _unit: PhantomData,
+        }
+    }
+}
+
+#[cfg(feature = "serde")]
+impl<'de, T, Src, Dst> serde::Deserialize<'de> for Transform2D<T, Src, Dst>
+    where T: serde::Deserialize<'de>
+{
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+        where D: serde::Deserializer<'de>
+    {
+        let (
+            m11, m12,
+            m21, m22,
+            m31, m32,
+        ) = try!(serde::Deserialize::deserialize(deserializer));
+        Ok(Transform2D {
+            m11, m12,
+            m21, m22,
+            m31, m32,
+            _unit: PhantomData
+        })
+    }
+}
+
+#[cfg(feature = "serde")]
+impl<T, Src, Dst> serde::Serialize for Transform2D<T, Src, Dst>
+    where T: serde::Serialize
+{
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+        where S: serde::Serializer
+    {
+        (
+            &self.m11, &self.m12,
+            &self.m21, &self.m22,
+            &self.m31, &self.m32,
+        ).serialize(serializer)
+    }
+}
+
+impl<T, Src, Dst> Eq for Transform2D<T, Src, Dst> where T: Eq {}
+
+impl<T, Src, Dst> PartialEq for Transform2D<T, Src, Dst>
+    where T: PartialEq
+{
+    fn eq(&self, other: &Self) -> bool {
+        self.m11 == other.m11 &&
+            self.m12 == other.m12 &&
+            self.m21 == other.m21 &&
+            self.m22 == other.m22 &&
+            self.m31 == other.m31 &&
+            self.m32 == other.m32
+    }
+}
+
+impl<T, Src, Dst> Hash for Transform2D<T, Src, Dst>
+    where T: Hash
+{
+    fn hash<H: ::core::hash::Hasher>(&self, h: &mut H) {
+        self.m11.hash(h);
+        self.m12.hash(h);
+        self.m21.hash(h);
+        self.m22.hash(h);
+        self.m31.hash(h);
+        self.m32.hash(h);
+    }
+}
+
+impl<T: Copy, Src, Dst> Transform2D<T, Src, Dst> {
     /// Create a transform specifying its matrix elements in row-major order.
     ///
     /// Beware: This library is written with the assumption that row vectors
     /// are being used. If your matrices use column vectors (i.e. transforming a vector
     /// is `T * v`), then please use `column_major`
     pub fn row_major(m11: T, m12: T, m21: T, m22: T, m31: T, m32: T) -> Self {
-        TypedTransform2D {
+        Transform2D {
             m11, m12,
             m21, m22,
             m31, m32,
@@ -73,7 +153,7 @@ impl<T: Copy, Src, Dst> TypedTransform2D<T, Src, Dst> {
     /// are being used. If your matrices use column vectors (i.e. transforming a vector
     /// is `T * v`), then please use `row_major`
     pub fn column_major(m11: T, m21: T, m31: T, m12: T, m22: T, m32: T) -> Self {
-        TypedTransform2D {
+        Transform2D {
             m11, m12,
             m21, m22,
             m31, m32,
@@ -150,7 +230,7 @@ impl<T: Copy, Src, Dst> TypedTransform2D<T, Src, Dst> {
     }
 
     /// Drop the units, preserving only the numeric value.
-    pub fn to_untyped(&self) -> Transform2D<T> {
+    pub fn to_untyped(&self) -> Transform2D<T, UnknownUnit, UnknownUnit> {
         Transform2D::row_major(
             self.m11, self.m12,
             self.m21, self.m22,
@@ -159,8 +239,8 @@ impl<T: Copy, Src, Dst> TypedTransform2D<T, Src, Dst> {
     }
 
     /// Tag a unitless value with units.
-    pub fn from_untyped(p: &Transform2D<T>) -> Self {
-        TypedTransform2D::row_major(
+    pub fn from_untyped(p: &Transform2D<T, UnknownUnit, UnknownUnit>) -> Self {
+        Transform2D::row_major(
             p.m11, p.m12,
             p.m21, p.m22,
             p.m31, p.m32
@@ -168,21 +248,21 @@ impl<T: Copy, Src, Dst> TypedTransform2D<T, Src, Dst> {
     }
 }
 
-impl<T0: NumCast + Copy, Src, Dst> TypedTransform2D<T0, Src, Dst> {
+impl<T0: NumCast + Copy, Src, Dst> Transform2D<T0, Src, Dst> {
     /// Cast from one numeric representation to another, preserving the units.
-    pub fn cast<T1: NumCast + Copy>(&self) -> TypedTransform2D<T1, Src, Dst> {
+    pub fn cast<T1: NumCast + Copy>(&self) -> Transform2D<T1, Src, Dst> {
         self.try_cast().unwrap()
     }
 
     /// Fallible cast from one numeric representation to another, preserving the units.
-    pub fn try_cast<T1: NumCast + Copy>(&self) -> Option<TypedTransform2D<T1, Src, Dst>> {
+    pub fn try_cast<T1: NumCast + Copy>(&self) -> Option<Transform2D<T1, Src, Dst>> {
         match (NumCast::from(self.m11), NumCast::from(self.m12),
                NumCast::from(self.m21), NumCast::from(self.m22),
                NumCast::from(self.m31), NumCast::from(self.m32)) {
             (Some(m11), Some(m12),
              Some(m21), Some(m22),
              Some(m31), Some(m32)) => {
-                Some(TypedTransform2D::row_major(
+                Some(Transform2D::row_major(
                     m11, m12,
                     m21, m22,
                     m31, m32
@@ -193,13 +273,13 @@ impl<T0: NumCast + Copy, Src, Dst> TypedTransform2D<T0, Src, Dst> {
     }
 }
 
-impl<T, Src, Dst> TypedTransform2D<T, Src, Dst>
+impl<T, Src, Dst> Transform2D<T, Src, Dst>
 where T: Copy +
          PartialEq +
          One + Zero {
     pub fn identity() -> Self {
         let (_0, _1) = (Zero::zero(), One::one());
-        TypedTransform2D::row_major(
+        Transform2D::row_major(
            _1, _0,
            _0, _1,
            _0, _0
@@ -210,11 +290,11 @@ where T: Copy +
     // while most consumers will probably want some sort of approximate
     // equivalence to deal with floating-point errors.
     fn is_identity(&self) -> bool {
-        *self == TypedTransform2D::identity()
+        *self == Transform2D::identity()
     }
 }
 
-impl<T, Src, Dst> TypedTransform2D<T, Src, Dst>
+impl<T, Src, Dst> Transform2D<T, Src, Dst>
 where T: Copy + Clone +
          Add<T, Output=T> +
          Mul<T, Output=T> +
@@ -228,9 +308,9 @@ where T: Copy + Clone +
     /// applies after self's transformation.
     ///
     /// Assuming row vectors, this is equivalent to self * mat
-    #[cfg_attr(feature = "unstable", must_use)]
-    pub fn post_mul<NewDst>(&self, mat: &TypedTransform2D<T, Dst, NewDst>) -> TypedTransform2D<T, Src, NewDst> {
-        TypedTransform2D::row_major(
+    #[must_use]
+    pub fn post_transform<NewDst>(&self, mat: &Transform2D<T, Dst, NewDst>) -> Transform2D<T, Src, NewDst> {
+        Transform2D::row_major(
             self.m11 * mat.m11 + self.m12 * mat.m21,
             self.m11 * mat.m12 + self.m12 * mat.m22,
             self.m21 * mat.m11 + self.m22 * mat.m21,
@@ -244,15 +324,17 @@ where T: Copy + Clone +
     /// applies before self's transformation.
     ///
     /// Assuming row vectors, this is equivalent to mat * self
-    #[cfg_attr(feature = "unstable", must_use)]
-    pub fn pre_mul<NewSrc>(&self, mat: &TypedTransform2D<T, NewSrc, Src>) -> TypedTransform2D<T, NewSrc, Dst> {
-        mat.post_mul(self)
+    #[inline]
+    #[must_use]
+    pub fn pre_transform<NewSrc>(&self, mat: &Transform2D<T, NewSrc, Src>) -> Transform2D<T, NewSrc, Dst> {
+        mat.post_transform(self)
     }
 
     /// Returns a translation transform.
+    #[inline]
     pub fn create_translation(x: T, y: T) -> Self {
          let (_0, _1): (T, T) = (Zero::zero(), One::one());
-         TypedTransform2D::row_major(
+         Transform2D::row_major(
             _1, _0,
             _0, _1,
              x,  y
@@ -260,21 +342,23 @@ where T: Copy + Clone +
     }
 
     /// Applies a translation after self's transformation and returns the resulting transform.
-    #[cfg_attr(feature = "unstable", must_use)]
-    pub fn post_translate(&self, v: TypedVector2D<T, Dst>) -> Self {
-        self.post_mul(&TypedTransform2D::create_translation(v.x, v.y))
+    #[inline]
+    #[must_use]
+    pub fn post_translate(&self, v: Vector2D<T, Dst>) -> Self {
+        self.post_transform(&Transform2D::create_translation(v.x, v.y))
     }
 
     /// Applies a translation before self's transformation and returns the resulting transform.
-    #[cfg_attr(feature = "unstable", must_use)]
-    pub fn pre_translate(&self, v: TypedVector2D<T, Src>) -> Self {
-        self.pre_mul(&TypedTransform2D::create_translation(v.x, v.y))
+    #[inline]
+    #[must_use]
+    pub fn pre_translate(&self, v: Vector2D<T, Src>) -> Self {
+        self.pre_transform(&Transform2D::create_translation(v.x, v.y))
     }
 
     /// Returns a scale transform.
     pub fn create_scale(x: T, y: T) -> Self {
         let _0 = Zero::zero();
-        TypedTransform2D::row_major(
+        Transform2D::row_major(
              x, _0,
             _0,  y,
             _0, _0
@@ -282,15 +366,17 @@ where T: Copy + Clone +
     }
 
     /// Applies a scale after self's transformation and returns the resulting transform.
-    #[cfg_attr(feature = "unstable", must_use)]
+    #[inline]
+    #[must_use]
     pub fn post_scale(&self, x: T, y: T) -> Self {
-        self.post_mul(&TypedTransform2D::create_scale(x, y))
+        self.post_transform(&Transform2D::create_scale(x, y))
     }
 
     /// Applies a scale before self's transformation and returns the resulting transform.
-    #[cfg_attr(feature = "unstable", must_use)]
+    #[inline]
+    #[must_use]
     pub fn pre_scale(&self, x: T, y: T) -> Self {
-        TypedTransform2D::row_major(
+        Transform2D::row_major(
             self.m11 * x, self.m12,
             self.m21,     self.m22 * y,
             self.m31,     self.m32
@@ -298,11 +384,12 @@ where T: Copy + Clone +
     }
 
     /// Returns a rotation transform.
+    #[inline]
     pub fn create_rotation(theta: Angle<T>) -> Self {
         let _0 = Zero::zero();
         let cos = theta.get().cos();
         let sin = theta.get().sin();
-        TypedTransform2D::row_major(
+        Transform2D::row_major(
             cos, _0 - sin,
             sin, cos,
              _0, _0
@@ -310,33 +397,37 @@ where T: Copy + Clone +
     }
 
     /// Applies a rotation after self's transformation and returns the resulting transform.
-    #[cfg_attr(feature = "unstable", must_use)]
+    #[inline]
+    #[must_use]
     pub fn post_rotate(&self, theta: Angle<T>) -> Self {
-        self.post_mul(&TypedTransform2D::create_rotation(theta))
+        self.post_transform(&Transform2D::create_rotation(theta))
     }
 
-    /// Applies a rotation after self's transformation and returns the resulting transform.
-    #[cfg_attr(feature = "unstable", must_use)]
+    /// Applies a rotation before self's transformation and returns the resulting transform.
+    #[inline]
+    #[must_use]
     pub fn pre_rotate(&self, theta: Angle<T>) -> Self {
-        self.pre_mul(&TypedTransform2D::create_rotation(theta))
+        self.pre_transform(&Transform2D::create_rotation(theta))
     }
 
     /// Returns the given point transformed by this transform.
     ///
     /// Assuming row vectors, this is equivalent to `p * self`
     #[inline]
-    #[cfg_attr(feature = "unstable", must_use)]
-    pub fn transform_point(&self, point: &TypedPoint2D<T, Src>) -> TypedPoint2D<T, Dst> {
-        TypedPoint2D::new(point.x * self.m11 + point.y * self.m21 + self.m31,
-                          point.x * self.m12 + point.y * self.m22 + self.m32)
+    #[must_use]
+    pub fn transform_point(&self, point: Point2D<T, Src>) -> Point2D<T, Dst> {
+        Point2D::new(
+            point.x * self.m11 + point.y * self.m21 + self.m31,
+            point.x * self.m12 + point.y * self.m22 + self.m32
+        )
     }
 
     /// Returns the given vector transformed by this matrix.
     ///
     /// Assuming row vectors, this is equivalent to `v * self`
     #[inline]
-    #[cfg_attr(feature = "unstable", must_use)]
-    pub fn transform_vector(&self, vec: &TypedVector2D<T, Src>) -> TypedVector2D<T, Dst> {
+    #[must_use]
+    pub fn transform_vector(&self, vec: Vector2D<T, Src>) -> Vector2D<T, Dst> {
         vec2(vec.x * self.m11 + vec.y * self.m21,
              vec.x * self.m12 + vec.y * self.m22)
     }
@@ -344,13 +435,15 @@ where T: Copy + Clone +
     /// Returns a rectangle that encompasses the result of transforming the given rectangle by this
     /// transform.
     #[inline]
-    #[cfg_attr(feature = "unstable", must_use)]
-    pub fn transform_rect(&self, rect: &TypedRect<T, Src>) -> TypedRect<T, Dst> {
-        TypedRect::from_points(&[
-            self.transform_point(&rect.origin),
-            self.transform_point(&rect.top_right()),
-            self.transform_point(&rect.bottom_left()),
-            self.transform_point(&rect.bottom_right()),
+    #[must_use]
+    pub fn transform_rect(&self, rect: &Rect<T, Src>) -> Rect<T, Dst> {
+        let min = rect.min();
+        let max = rect.max();
+        Rect::from_points(&[
+            self.transform_point(min),
+            self.transform_point(max),
+            self.transform_point(point2(max.x, min.y)),
+            self.transform_point(point2(min.x, max.y)),
         ])
     }
 
@@ -360,8 +453,8 @@ where T: Copy + Clone +
     }
 
     /// Returns the inverse transform if possible.
-    #[cfg_attr(feature = "unstable", must_use)]
-    pub fn inverse(&self) -> Option<TypedTransform2D<T, Dst, Src>> {
+    #[must_use]
+    pub fn inverse(&self) -> Option<Transform2D<T, Dst, Src>> {
         let det = self.determinant();
 
         let _0: T = Zero::zero();
@@ -372,7 +465,7 @@ where T: Copy + Clone +
         }
 
         let inv_det = _1 / det;
-        Some(TypedTransform2D::row_major(
+        Some(Transform2D::row_major(
             inv_det * self.m22,
             inv_det * (_0 - self.m12),
             inv_det * (_0 - self.m21),
@@ -384,8 +477,8 @@ where T: Copy + Clone +
 
     /// Returns the same transform with a different destination unit.
     #[inline]
-    pub fn with_destination<NewDst>(&self) -> TypedTransform2D<T, Src, NewDst> {
-        TypedTransform2D::row_major(
+    pub fn with_destination<NewDst>(&self) -> Transform2D<T, Src, NewDst> {
+        Transform2D::row_major(
             self.m11, self.m12,
             self.m21, self.m22,
             self.m31, self.m32,
@@ -394,8 +487,8 @@ where T: Copy + Clone +
 
     /// Returns the same transform with a different source unit.
     #[inline]
-    pub fn with_source<NewSrc>(&self) -> TypedTransform2D<T, NewSrc, Dst> {
-        TypedTransform2D::row_major(
+    pub fn with_source<NewSrc>(&self) -> Transform2D<T, NewSrc, Dst> {
+        Transform2D::row_major(
             self.m11, self.m12,
             self.m21, self.m22,
             self.m31, self.m32,
@@ -403,7 +496,7 @@ where T: Copy + Clone +
     }
 }
 
-impl <T, Src, Dst> TypedTransform2D<T, Src, Dst>
+impl <T, Src, Dst> Transform2D<T, Src, Dst>
 where T: Copy + Clone +
          Add<T, Output=T> +
          Sub<T, Output=T> +
@@ -414,13 +507,13 @@ where T: Copy + Clone +
          Trig +
          One + Zero {
     /// Create a 3D transform from the current transform
-    pub fn to_3d(&self) -> TypedTransform3D<T, Src, Dst> {
-        TypedTransform3D::row_major_2d(self.m11, self.m12, self.m21, self.m22, self.m31, self.m32)
+    pub fn to_3d(&self) -> Transform3D<T, Src, Dst> {
+        Transform3D::row_major_2d(self.m11, self.m12, self.m21, self.m22, self.m31, self.m32)
     }
 
 }
 
-impl <T, Src, Dst> Default for TypedTransform2D<T, Src, Dst>
+impl <T, Src, Dst> Default for Transform2D<T, Src, Dst>
     where T: Copy + PartialEq + One + Zero
 {
     fn default() -> Self {
@@ -428,7 +521,7 @@ impl <T, Src, Dst> Default for TypedTransform2D<T, Src, Dst>
     }
 }
 
-impl<T: ApproxEq<T>, Src, Dst> TypedTransform2D<T, Src, Dst> {
+impl<T: ApproxEq<T>, Src, Dst> Transform2D<T, Src, Dst> {
     pub fn approx_eq(&self, other: &Self) -> bool {
         self.m11.approx_eq(&other.m11) && self.m12.approx_eq(&other.m12) &&
         self.m21.approx_eq(&other.m21) && self.m22.approx_eq(&other.m22) &&
@@ -436,7 +529,7 @@ impl<T: ApproxEq<T>, Src, Dst> TypedTransform2D<T, Src, Dst> {
     }
 }
 
-impl<T: Copy + fmt::Debug, Src, Dst> fmt::Debug for TypedTransform2D<T, Src, Dst>
+impl<T: Copy + fmt::Debug, Src, Dst> fmt::Debug for Transform2D<T, Src, Dst>
 where T: Copy + fmt::Debug +
          PartialEq +
          One + Zero {
@@ -450,9 +543,9 @@ where T: Copy + fmt::Debug +
 }
 
 #[cfg(feature = "mint")]
-impl<T, Src, Dst> From<mint::RowMatrix3x2<T>> for TypedTransform2D<T, Src, Dst> {
+impl<T, Src, Dst> From<mint::RowMatrix3x2<T>> for Transform2D<T, Src, Dst> {
     fn from(m: mint::RowMatrix3x2<T>) -> Self {
-        TypedTransform2D {
+        Transform2D {
             m11: m.x.x, m12: m.x.y,
             m21: m.y.x, m22: m.y.y,
             m31: m.z.x, m32: m.z.y,
@@ -461,7 +554,7 @@ impl<T, Src, Dst> From<mint::RowMatrix3x2<T>> for TypedTransform2D<T, Src, Dst>
     }
 }
 #[cfg(feature = "mint")]
-impl<T, Src, Dst> Into<mint::RowMatrix3x2<T>> for TypedTransform2D<T, Src, Dst> {
+impl<T, Src, Dst> Into<mint::RowMatrix3x2<T>> for Transform2D<T, Src, Dst> {
     fn into(self) -> mint::RowMatrix3x2<T> {
         mint::RowMatrix3x2 {
             x: mint::Vector2 { x: self.m11, y: self.m12 },
@@ -475,15 +568,14 @@ impl<T, Src, Dst> Into<mint::RowMatrix3x2<T>> for TypedTransform2D<T, Src, Dst>
 #[cfg(test)]
 mod test {
     use super::*;
+    use default;
     use approxeq::ApproxEq;
-    use point::Point2D;
-    use Angle;
     #[cfg(feature = "mint")]
     use mint;
 
     use core::f32::consts::FRAC_PI_2;
 
-    type Mat = Transform2D<f32>;
+    type Mat = default::Transform2D<f32>;
 
     fn rad(v: f32) -> Angle<f32> { Angle::radians(v) }
 
@@ -495,9 +587,9 @@ mod test {
         assert_eq!(t1, t2);
         assert_eq!(t1, t3);
 
-        assert_eq!(t1.transform_point(&Point2D::new(1.0, 1.0)), Point2D::new(2.0, 3.0));
+        assert_eq!(t1.transform_point(Point2D::new(1.0, 1.0)), Point2D::new(2.0, 3.0));
 
-        assert_eq!(t1.post_mul(&t1), Mat::create_translation(2.0, 4.0));
+        assert_eq!(t1.post_transform(&t1), Mat::create_translation(2.0, 4.0));
     }
 
     #[test]
@@ -508,9 +600,9 @@ mod test {
         assert_eq!(r1, r2);
         assert_eq!(r1, r3);
 
-        assert!(r1.transform_point(&Point2D::new(1.0, 2.0)).approx_eq(&Point2D::new(2.0, -1.0)));
+        assert!(r1.transform_point(Point2D::new(1.0, 2.0)).approx_eq(&Point2D::new(2.0, -1.0)));
 
-        assert!(r1.post_mul(&r1).approx_eq(&Mat::create_rotation(rad(FRAC_PI_2*2.0))));
+        assert!(r1.post_transform(&r1).approx_eq(&Mat::create_rotation(rad(FRAC_PI_2*2.0))));
     }
 
     #[test]
@@ -521,7 +613,7 @@ mod test {
         assert_eq!(s1, s2);
         assert_eq!(s1, s3);
 
-        assert!(s1.transform_point(&Point2D::new(2.0, 2.0)).approx_eq(&Point2D::new(4.0, 6.0)));
+        assert!(s1.transform_point(Point2D::new(2.0, 2.0)).approx_eq(&Point2D::new(4.0, 6.0)));
     }
 
     #[test]
@@ -550,14 +642,14 @@ mod test {
     pub fn test_inverse_scale() {
         let m1 = Mat::create_scale(1.5, 0.3);
         let m2 = m1.inverse().unwrap();
-        assert!(m1.pre_mul(&m2).approx_eq(&Mat::identity()));
+        assert!(m1.pre_transform(&m2).approx_eq(&Mat::identity()));
     }
 
     #[test]
     pub fn test_inverse_translate() {
         let m1 = Mat::create_translation(-132.0, 0.3);
         let m2 = m1.inverse().unwrap();
-        assert!(m1.pre_mul(&m2).approx_eq(&Mat::identity()));
+        assert!(m1.pre_transform(&m2).approx_eq(&Mat::identity()));
     }
 
     #[test]
@@ -568,8 +660,8 @@ mod test {
 
     #[test]
     pub fn test_pre_post() {
-        let m1 = Transform2D::identity().post_scale(1.0, 2.0).post_translate(vec2(1.0, 2.0));
-        let m2 = Transform2D::identity().pre_translate(vec2(1.0, 2.0)).pre_scale(1.0, 2.0);
+        let m1 = default::Transform2D::identity().post_scale(1.0, 2.0).post_translate(vec2(1.0, 2.0));
+        let m2 = default::Transform2D::identity().pre_translate(vec2(1.0, 2.0)).pre_scale(1.0, 2.0);
         assert!(m1.approx_eq(&m2));
 
         let r = Mat::create_rotation(rad(FRAC_PI_2));
@@ -577,25 +669,25 @@ mod test {
 
         let a = Point2D::new(1.0, 1.0);
 
-        assert!(r.post_mul(&t).transform_point(&a).approx_eq(&Point2D::new(3.0, 2.0)));
-        assert!(t.post_mul(&r).transform_point(&a).approx_eq(&Point2D::new(4.0, -3.0)));
-        assert!(t.post_mul(&r).transform_point(&a).approx_eq(&r.transform_point(&t.transform_point(&a))));
+        assert!(r.post_transform(&t).transform_point(a).approx_eq(&Point2D::new(3.0, 2.0)));
+        assert!(t.post_transform(&r).transform_point(a).approx_eq(&Point2D::new(4.0, -3.0)));
+        assert!(t.post_transform(&r).transform_point(a).approx_eq(&r.transform_point(t.transform_point(a))));
 
-        assert!(r.pre_mul(&t).transform_point(&a).approx_eq(&Point2D::new(4.0, -3.0)));
-        assert!(t.pre_mul(&r).transform_point(&a).approx_eq(&Point2D::new(3.0, 2.0)));
-        assert!(t.pre_mul(&r).transform_point(&a).approx_eq(&t.transform_point(&r.transform_point(&a))));
+        assert!(r.pre_transform(&t).transform_point(a).approx_eq(&Point2D::new(4.0, -3.0)));
+        assert!(t.pre_transform(&r).transform_point(a).approx_eq(&Point2D::new(3.0, 2.0)));
+        assert!(t.pre_transform(&r).transform_point(a).approx_eq(&t.transform_point(r.transform_point(a))));
     }
 
     #[test]
     fn test_size_of() {
         use core::mem::size_of;
-        assert_eq!(size_of::<Transform2D<f32>>(), 6*size_of::<f32>());
-        assert_eq!(size_of::<Transform2D<f64>>(), 6*size_of::<f64>());
+        assert_eq!(size_of::<default::Transform2D<f32>>(), 6*size_of::<f32>());
+        assert_eq!(size_of::<default::Transform2D<f64>>(), 6*size_of::<f64>());
     }
 
     #[test]
     pub fn test_is_identity() {
-        let m1 = Transform2D::identity();
+        let m1 = default::Transform2D::identity();
         assert!(m1.is_identity());
         let m2 = m1.post_translate(vec2(0.1, 0.0));
         assert!(!m2.is_identity());
@@ -606,7 +698,7 @@ mod test {
         // Translation does not apply to vectors.
         let m1 = Mat::create_translation(1.0, 1.0);
         let v1 = vec2(10.0, -10.0);
-        assert_eq!(v1, m1.transform_vector(&v1));
+        assert_eq!(v1, m1.transform_vector(v1));
     }
 
     #[cfg(feature = "mint")]
